package org.beigesoft.holder;

/*
 * Copyright (c) 2018 Beigesoft™
 *
 * Licensed under the GNU General Public License (GPL), Version 2.0
 * (the "License");
 * you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at
 *
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html
 */

import java.util.Map;

import org.beigesoft.service.ISrvOrm;
import org.beigesoft.persistable.MatchForeign;
import org.beigesoft.persistable.MatchForeignLine;

/**
 * <p>Retriever Match Foreign from database.</p>
 *
 * @param <RS> platform dependent RDBMS recordset
 * @author Yury Demidenko
 */
public class HldMatchForeign<RS> implements IHldMatchForeign {

  /**
   * <p>ORM service.</p>
   **/
  private ISrvOrm<RS> srvOrm;

  /**
   * <p>Get match foreign  given ID.</p>
   * @param pReqVars additional param
   * @param pId ID
   * @return match foreign
   * @throws Exception an Exception
   **/
  public final MatchForeign getFor(final Map<String, Object> pReqVars,
    final Long pId) throws Exception {
    MatchForeign mf = getSrvOrm().retrieveEntityById(pReqVars,
      MatchForeign.class, pId);
    mf.setItsLines(getSrvOrm().retrieveListWithConditions(pReqVars,
      MatchForeignLine.class, "where ITSOWNER=" + pId));
    return mf;
  }

  //Simple getters and setters:
  /**
   * <p>Getter for srvOrm.</p>
   * @return ISrvOrm<RS>
   **/
  public final ISrvOrm<RS> getSrvOrm() {
    return this.srvOrm;
  }

  /**
   * <p>Setter for srvOrm.</p>
   * @param pSrvOrm reference
   **/
  public final void setSrvOrm(final ISrvOrm<RS> pSrvOrm) {
    this.srvOrm = pSrvOrm;
  }
}
