package org.beigesoft.persistable;

/*
 * Copyright (c) 2018 Beigesoft™
 *
 * Licensed under the GNU General Public License (GPL), Version 2.0
 * (the "License");
 * you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at
 *
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html
 */

import org.beigesoft.model.IHasIdName;
import org.beigesoft.model.IHasIdLongVersionName;
import org.beigesoft.model.IHasVersion;
import org.beigesoft.model.AEditable;

/**
 * <p>
 * Model of I18N name. Beige-ORM does not support generic fields,
 * so internationalized things IHasName should be in extending class.
 * And so does language cause it's used in the ID.
 * </p>
 *
 * @param <T> item type
 * @param <ID> ID type
 * @author Yury Demidenko
 */
public abstract class
  AI18nName<T extends IHasIdLongVersionName, ID extends AI18nNameId<T>>
    extends AEditable implements IHasVersion, IHasIdName<ID> {

  /**
   * <p>Name in the language.</p>
   **/
  private String itsName;

  /**
   * <p>Version to check dirty and/or replication.</p>
   **/
  private Long itsVersion;

  /**
   * <p>Getter for hasName.</p>
   * @return IHasName
   **/
  public abstract T getHasName();

  /**
   * <p>Setter for hasName.</p>
   * @param pHasName reference
   **/
  public abstract void setHasName(T pHasName);

  /**
   * <p>Getter for lang.</p>
   * @return Languages
   **/
  public abstract Languages getLang();

  /**
   * <p>Setter for lang.</p>
   * @param pLang reference
   **/
  public abstract void setLang(Languages pLang);

  /**
   * <p>Geter for itsVersion.</p>
   * @return Long
   **/
  @Override
  public final Long getItsVersion() {
    return this.itsVersion;
  }

  /**
   * <p>Setter for itsVersion.</p>
   * @param pItsVersion reference
   **/
  @Override
  public final void setItsVersion(final Long pItsVersion) {
    this.itsVersion = pItsVersion;
  }

  /**
   * <p>Geter for itsName.</p>
   * @return String
   **/
  @Override
  public final String getItsName() {
    return this.itsName;
  }

  /**
   * <p>Setter for itsName.</p>
   * @param pItsName reference
   **/
  @Override
  public final void setItsName(final String pItsName) {
    this.itsName = pItsName;
  }
}
