package org.beigesoft.persistable;

/*
 * Copyright (c) 2018 Beigesoft™
 *
 * Licensed under the GNU General Public License (GPL), Version 2.0
 * (the "License");
 * you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at
 *
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html
 */

import org.beigesoft.model.ECsvColumnType;
import org.beigesoft.model.IOwned;

/**
 * <p>Persistable model of column CSV method, i.e. how to import/export
 * data from/to column, common rules.</p>
 *
 * @author Yury Demidenko
 */
public class CsvColumn extends AHasNameIdLongVersion
  implements IOwned<CsvMethod> {

  /**
   * <p>CSV method.</p>
   **/
  private CsvMethod itsOwner;

  /**
   * <p>Column data type.</p>
   **/
  private ECsvColumnType itsType;

  /**
   * <p>Column index, from 1.</p>
   **/
  private Integer itsIndex;

  /**
   * <p>Column index, from 1 in source file
   * cause some columns might be omitted.</p>
   **/
  private Integer sourceIndex;

  /**
   * <p>Column's text braced with delimiters, null default,
   * e.g. quoted - "taxes, penalty and other fees are included".</p>
   **/
  private String textDelimiter = null;

  /**
   * <p>Column's data format, null default, e.g. "dd/MM/yyyy" for Date,
   * "space,comma" for number "123 31,78EUR", "true,false" for Boolean.</p>
   **/
  private String dataFormat = null;

  //rules to write:
  /**
   * <p>If used, constant for default value for all rows,
   * e.g. "1" for column "isTaxIncludedInPrice".</p>
   **/
  private String constValue;

  /**
   * <p>If used, index of data column, starts from 1.</p>
   **/
  private Integer dataIndex;

  /**
   * <p>If used, comma separated fields names
   * trough destination field, e.g.
   * "itsCategory,itsId"
   * to retrieve product.getItsCategory().getItsId() from column
   * with given dataIndex.</p>
   **/
  private String fieldPath;

  /**
   * <p>If used, match foreign method.</p>
   **/
  private MatchForeign matchForeign;

  /**
   * <p>Getter for itsOwner.</p>
   * @return CsvMethod
   **/
  @Override
  public final CsvMethod getItsOwner() {
    return this.itsOwner;
  }

  /**
   * <p>Setter for itsOwner.</p>
   * @param pItsOwner reference
   **/
  @Override
  public final void setItsOwner(final CsvMethod pItsOwner) {
    this.itsOwner = pItsOwner;
  }

  //Simple getters and setters:
  /**
   * <p>Getter for itsType.</p>
   * @return ECsvColumnType
   **/
  public final ECsvColumnType getItsType() {
    return this.itsType;
  }

  /**
   * <p>Setter for itsType.</p>
   * @param pItsType reference
   **/
  public final void setItsType(final ECsvColumnType pItsType) {
    this.itsType = pItsType;
  }

  /**
   * <p>Getter for itsIndex.</p>
   * @return Integer
   **/
  public final Integer getItsIndex() {
    return this.itsIndex;
  }

  /**
   * <p>Setter for itsIndex.</p>
   * @param pItsIndex reference
   **/
  public final void setItsIndex(final Integer pItsIndex) {
    this.itsIndex = pItsIndex;
  }

  /**
   * <p>Getter for sourceIndex.</p>
   * @return Integer
   **/
  public final Integer getSourceIndex() {
    return this.sourceIndex;
  }

  /**
   * <p>Setter for sourceIndex.</p>
   * @param pSourceIndex reference
   **/
  public final void setSourceIndex(final Integer pSourceIndex) {
    this.sourceIndex = pSourceIndex;
  }

  /**
   * <p>Getter for textDelimiter.</p>
   * @return String
   **/
  public final String getTextDelimiter() {
    return this.textDelimiter;
  }

  /**
   * <p>Setter for textDelimiter.</p>
   * @param pTextDelimiter reference
   **/
  public final void setTextDelimiter(final String pTextDelimiter) {
    this.textDelimiter = pTextDelimiter;
  }

  /**
   * <p>Getter for dataFormat.</p>
   * @return String
   **/
  public final String getDataFormat() {
    return this.dataFormat;
  }

  /**
   * <p>Setter for dataFormat.</p>
   * @param pDataFormat reference
   **/
  public final void setDataFormat(final String pDataFormat) {
    this.dataFormat = pDataFormat;
  }

  /**
   * <p>Getter for constValue.</p>
   * @return String
   **/
  public final String getConstValue() {
    return this.constValue;
  }

  /**
   * <p>Setter for constValue.</p>
   * @param pConstValue reference
   **/
  public final void setConstValue(final String pConstValue) {
    this.constValue = pConstValue;
  }

  /**
   * <p>Getter for dataIndex.</p>
   * @return Integer
   **/
  public final Integer getDataIndex() {
    return this.dataIndex;
  }

  /**
   * <p>Setter for dataIndex.</p>
   * @param pDataIndex reference
   **/
  public final void setDataIndex(final Integer pDataIndex) {
    this.dataIndex = pDataIndex;
  }

  /**
   * <p>Getter for fieldPath.</p>
   * @return String
   **/
  public final String getFieldPath() {
    return this.fieldPath;
  }

  /**
   * <p>Setter for fieldPath.</p>
   * @param pFieldPath reference
   **/
  public final void setFieldPath(final String pFieldPath) {
    this.fieldPath = pFieldPath;
  }

  /**
   * <p>Getter for matchForeign.</p>
   * @return MatchForeign
   **/
  public final MatchForeign getMatchForeign() {
    return this.matchForeign;
  }

  /**
   * <p>Setter for matchForeign.</p>
   * @param pMatchForeign reference
   **/
  public final void setMatchForeign(final MatchForeign pMatchForeign) {
    this.matchForeign = pMatchForeign;
  }
}
