package org.beigesoft.persistable;

/*
 * Copyright (c) 2018 Beigesoft™
 *
 * Licensed under the GNU General Public License (GPL), Version 2.0
 * (the "License");
 * you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at
 *
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html
 */

import java.util.List;

/**
 * <p>Persistable model of basic CSV method, i.e. how to import/export
 * data from/to CSV file, common rules.</p>
 *
 * @author Yury Demidenko
 */
public class CsvMethod extends AHasNameIdLongVersion {

  /**
   * <p>Charset encoding name, "UTF-8" default.</p>
   **/
  private String charsetName = "UTF-8";

  /**
   * <p>Column separator, comma default.</p>
   **/
  private String columnSep = ",";

  /**
   * <p>The first row contains of columns names, false default.</p>
   **/
  private Boolean hasHeader = false;

  /**
   * <p>If used, name to get ICsvDataRetriever.</p>
   **/
  private String retrieverName;

  /**
   * <p>Columns all or useful to read.</p>
   **/
  private List<CsvColumn> columns;

  //Simple getters and setters:

  /**
   * <p>Getter for charsetName.</p>
   * @return String
   **/
  public final String getCharsetName() {
    return this.charsetName;
  }

  /**
   * <p>Setter for charsetName.</p>
   * @param pCharsetName reference
   **/
  public final void setCharsetName(final String pCharsetName) {
    this.charsetName = pCharsetName;
  }

  /**
   * <p>Getter for columnSep.</p>
   * @return String
   **/
  public final String getColumnSep() {
    return this.columnSep;
  }

  /**
   * <p>Setter for columnSep.</p>
   * @param pColumnSep reference
   **/
  public final void setColumnSep(final String pColumnSep) {
    this.columnSep = pColumnSep;
  }

  /**
   * <p>Getter for hasHeader.</p>
   * @return Boolean
   **/
  public final Boolean getHasHeader() {
    return this.hasHeader;
  }

  /**
   * <p>Setter for hasHeader.</p>
   * @param pHasHeader reference
   **/
  public final void setHasHeader(final Boolean pHasHeader) {
    this.hasHeader = pHasHeader;
  }

  /**
   * <p>Getter for columns.</p>
   * @return List<CsvColumn>
   **/
  public final List<CsvColumn> getColumns() {
    return this.columns;
  }

  /**
   * <p>Setter for columns.</p>
   * @param pColumns reference
   **/
  public final void setColumns(final List<CsvColumn> pColumns) {
    this.columns = pColumns;
  }


  /**
   * <p>Getter for retrieverName.</p>
   * @return String
   **/
  public final String getRetrieverName() {
    return this.retrieverName;
  }

  /**
   * <p>Setter for retrieverName.</p>
   * @param pRetrieverName reference
   **/
  public final void setRetrieverName(final String pRetrieverName) {
    this.retrieverName = pRetrieverName;
  }
}
