package org.beigesoft.persistable;

/*
 * Copyright (c) 2018 Beigesoft ™
 *
 * Licensed under the GNU General Public License (GPL), Version 2.0
 * (the "License");
 * you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at
 *
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html
 */

import org.beigesoft.model.AEditable;
import org.beigesoft.model.IHasId;
import org.beigesoft.model.IHasVersion;

/**
 * <p>
 * Model of I18N preferences for a country.
 * </p>
 *
 * @author Yury Demidenko
 */
public class LangPreferences extends AEditable
  implements IHasId<IdLangPreferences>, IHasVersion {

  /**
   * <p>ID.</p>
   **/
  private IdLangPreferences itsId = new IdLangPreferences();

  /**
   * <p>Country.</p>
   **/
  private Countries country;

  /**
   * <p>Language.</p>
   **/
  private Languages lang;

  /**
   * <p>Decimal Separator.</p>
   **/
  private DecimalSeparator decimalSep;

  /**
   * <p>Decimal Group Separator.</p>
   **/
  private DecimalGroupSeparator decimalGroupSep;

  /**
   * <p>Digits in group e.g. 2 for "1 21 23,90", 3 default.</p>
   **/
  private Integer digitsInGroup = 3;

  /**
   * <p>Is default.</p>
   **/
  private Boolean isDefault;

  /**
   * <p>Version to check dirty or replication.</p>
   **/
  private Long itsVersion;

  /**
   * <p>Geter for itsVersion.</p>
   * @return Long
   **/
  @Override
  public final Long getItsVersion() {
    return this.itsVersion;
  }

  /**
   * <p>Setter for itsVersion.</p>
   * @param pItsVersion reference
   **/
  @Override
  public final void setItsVersion(final Long pItsVersion) {
    this.itsVersion = pItsVersion;
  }

  /**
   * <p>Getter for itsId.</p>
   * @return IdLangPreferences
   **/
  @Override
  public final IdLangPreferences getItsId() {
    return this.itsId;
  }

  /**
   * <p>Setter for itsId.</p>
   * @param pItsId reference
   **/
  @Override
  public final void setItsId(final IdLangPreferences pItsId) {
    this.itsId = pItsId;
    if (this.itsId == null) {
      this.lang = null;
      this.country = null;
    } else {
      this.lang = this.itsId.getLang();
      this.country = this.itsId.getCountry();
    }
  }

  /**
   * <p>Setter for country.</p>
   * @param pCountry reference
   **/
  public final void setCountry(final Countries pCountry) {
    this.country = pCountry;
    if (this.itsId == null) {
      this.itsId = new IdLangPreferences();
    }
    this.itsId.setCountry(this.country);
  }

  /**
   * <p>Setter for lang.</p>
   * @param pLang reference
   **/
  public final void setLang(final Languages pLang) {
    this.lang = pLang;
    if (this.itsId == null) {
      this.itsId = new IdLangPreferences();
    }
    this.itsId.setLang(this.lang);
  }

  //Simple getters and setters:
  /**
   * <p>Getter for country.</p>
   * @return Countries
   **/
  public final Countries getCountry() {
    return this.country;
  }

  /**
   * <p>Getter for lang.</p>
   * @return Languages
   **/
  public final Languages getLang() {
    return this.lang;
  }

  /**
   * <p>Getter for decimalSep.</p>
   * @return DecimalSeparator
   **/
  public final DecimalSeparator getDecimalSep() {
    return this.decimalSep;
  }

  /**
   * <p>Setter for decimalSep.</p>
   * @param pDecimalSep reference
   **/
  public final void setDecimalSep(final DecimalSeparator pDecimalSep) {
    this.decimalSep = pDecimalSep;
  }

  /**
   * <p>Getter for decimalGroupSep.</p>
   * @return DecimalGroupSeparator
   **/
  public final DecimalGroupSeparator getDecimalGroupSep() {
    return this.decimalGroupSep;
  }

  /**
   * <p>Setter for decimalGroupSep.</p>
   * @param pDecimalGroupSep reference
   **/
  public final void setDecimalGroupSep(
    final DecimalGroupSeparator pDecimalGroupSep) {
    this.decimalGroupSep = pDecimalGroupSep;
  }

  /**
   * <p>Getter for isDefault.</p>
   * @return Boolean
   **/
  public final Boolean getIsDefault() {
    return this.isDefault;
  }

  /**
   * <p>Setter for isDefault.</p>
   * @param pIsDefault reference
   **/
  public final void setIsDefault(final Boolean pIsDefault) {
    this.isDefault = pIsDefault;
  }

  /**
   * <p>Getter for digitsInGroup.</p>
   * @return Integer
   **/
  public final Integer getDigitsInGroup() {
    return this.digitsInGroup;
  }

  /**
   * <p>Setter for digitsInGroup.</p>
   * @param pDigitsInGroup reference
   **/
  public final void setDigitsInGroup(final Integer pDigitsInGroup) {
    this.digitsInGroup = pDigitsInGroup;
  }
}
