package org.beigesoft.processor;

/*
 * Copyright (c) 2018 Beigesoft™
 *
 * Licensed under the GNU General Public License (GPL), Version 2.0
 * (the "License");
 * you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at
 *
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html
 */

import java.util.Map;

import org.beigesoft.exception.ExceptionWithCode;
import org.beigesoft.model.IRequestData;
import org.beigesoft.service.IProcessor;
import org.beigesoft.service.ICsvDataRetriever;

/**
 * <p>Service that retrieves CSV sample data row.</p>
 *
 * @author Yury Demidenko
 */
public class PrcCsvSampleDataRow implements IProcessor {

  /**
   * <p>Retrievers map.</p>
   **/
  private Map<String, ICsvDataRetriever> retrievers;

  /**
   * <p>Process entity request.</p>
   * @param pAddParam additional param
   * @param pRequestData Request Data
   * @throws Exception - an exception
   **/
  @Override
  public final void process(final Map<String, Object> pAddParam,
    final IRequestData pRequestData) throws Exception {
    String retrieverName = pRequestData.getParameter("nmRet");
    ICsvDataRetriever ret = this.retrievers.get(retrieverName);
    if (ret == null) {
      throw new ExceptionWithCode(ExceptionWithCode.WRONG_PARAMETER,
        "Can't find retriever " + retrieverName);
    }
    pRequestData.setAttribute("dataTree", ret.getSampleDataRow(pAddParam));
  }

  //Simple getters and setters:

  /**
   * <p>Getter for retrievers.</p>
   * @return Map<String, ICsvDataRetriever>
   **/
  public final Map<String, ICsvDataRetriever> getRetrievers() {
    return this.retrievers;
  }

  /**
   * <p>Setter for retrievers.</p>
   * @param pRetrievers reference
   **/
  public final void setRetrievers(
    final Map<String, ICsvDataRetriever> pRetrievers) {
    this.retrievers = pRetrievers;
  }
}
