package org.beigesoft.service;

/*
 * Copyright (c) 2018 Beigesoft ™
 *
 * Licensed under the GNU General Public License (GPL), Version 2.0
 * (the "License");
 * you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at
 *
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html
 */

/**
 * <p>Abstraction of service that prints a number according given
 * decimal separator, decimal group separator, decimal places after dot,
 * and digits in the group. The number is passed as String with dot decimal
 * separator (or without it) and no group separator,
 * e.g. "12346734.123" or "12667711". This service never does rounding,
 * so if you pass number "1234.2356" and decimal places after dot 3,
 * then result will be "1,234.235". For null or empty string result will be
 * empty string. </p>
 *
 * @author Yury Demidenko
 */
public interface ISrvNumberToString {

  /**
   * <p>Prints number formatted by given decimal separator, decimal group
   * separator and decimal places after dot. Digits in group is 3.</p>
   * @param pNumber e.g. "12146678.12"
   * @param pDigSep decimal separator, e.g. ","
   * @param pDigGrSep decimal group separator, e.g. " "
   * @param pDecPlAfDot decimal places after dot, e.g. 2
   * @return String internationalized number, e.g. "12 146 678,12"
   **/
  String print(String pNumber, String pDigSep, String pDigGrSep,
    Integer pDecPlAfDot);

  /**
   * <p>Prints number formatted by given decimal separator, decimal group
   * separator, decimal places after dot and digits in group.</p>
   * @param pNumber e.g. "12146678.12"
   * @param pDigSep decimal separator, e.g. ","
   * @param pDigGrSep decimal group separator, e.g. " "
   * @param pDecPlAfDot decimal places after dot, e.g. 2
   * @param pDigitsInGroup Digits in group, e.g. 3
   * @return String internationalized number, e.g. "12 146 678,12"
   **/
  String print(String pNumber, String pDigSep, String pDigGrSep,
    Integer pDecPlAfDot, Integer pDigitsInGroup);
}
