package org.beigesoft.service;

/*
 * Copyright (c) 2018 Beigesoft™
 *
 * Licensed under the GNU General Public License (GPL), Version 2.0
 * (the "License");
 * you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at
 *
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html
 */

import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;
import java.util.Date;
import java.math.BigDecimal;
import java.io.StringReader;
import java.io.File;
import java.io.Reader;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.nio.charset.Charset;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import org.junit.Test;

import org.beigesoft.log.LoggerSimple;

import org.beigesoft.persistable.CsvMethod;
import org.beigesoft.persistable.CsvColumn;

/**
 * <p>CsvReader test.</p>
 *
 * @author Yury Demidenko
 */
public class CsvReaderTest {

  //LoggerSimple logger = new LoggerSimple();

  CsvReader csvReader = new CsvReader();

  CsvMethod csvMethod;

  public CsvReaderTest() {
    this.csvMethod = new CsvMethod();
    this.csvMethod.setCharsetName("ASCII");
    this.csvMethod.setColumns(new ArrayList<CsvColumn>());
    CsvColumn colDate = new CsvColumn();
    colDate.setItsIndex(1);
    colDate.setSourceIndex(1);
    this.csvMethod.getColumns().add(colDate);
    CsvColumn colDesc = new CsvColumn();
    colDesc.setItsIndex(2);
    colDesc.setSourceIndex(2);
    colDesc.setTextDelimiter("\"");
    this.csvMethod.getColumns().add(colDesc);
    CsvColumn colAm = new CsvColumn();
    colAm.setItsIndex(3);
    colAm.setSourceIndex(4);
    this.csvMethod.getColumns().add(colAm);
    CsvColumn colCause = new CsvColumn();
    colCause.setItsIndex(4);
    colCause.setSourceIndex(5);
    this.csvMethod.getColumns().add(colCause);
  }

  @Test
  public void test1() {
    String filePath = "src" + File.separator + "test" + File.separator
      + "resources" + File.separator + "bnk-stm-lf.csv";
    read1(filePath);
    filePath = "src" + File.separator + "test" + File.separator
      + "resources" + File.separator + "bnk-stm-crlf.csv";
    read1(filePath);
    filePath = "src" + File.separator + "test" + File.separator
      + "resources" + File.separator + "bnk-stm-cr.csv";
    read1(filePath);
  }

  @Test
  public void test2() {
    String filePath = "src" + File.separator + "test" + File.separator
      + "resources" + File.separator + "bnk-stm-head-lf.csv";
    read2(filePath);
  }

  public void read1(String pFilePath) {
    List<String> row1 = null;
    List<String> row2 = null;
    InputStreamReader reader = null;
    try {
      reader = new InputStreamReader(new FileInputStream(pFilePath),
        Charset.forName(this.csvMethod.getCharsetName()).newDecoder());
      row1 = this.csvReader.readNextRow(null, reader, this.csvMethod);
      row2 = this.csvReader.readNextRow(null, reader, this.csvMethod);
    } catch (Exception e) {
      e.printStackTrace();
    } finally {
      if (reader != null) {
        try {
          reader.close();
        } catch (Exception e) {
          e.printStackTrace();
        }
      }
    }
    System.out.println("row1: ");
    for (String v : row1) {
      System.out.print(v + ",");
    }
    System.out.println("");
    assertEquals("01/01/2018", row1.get(0)); 
    assertEquals("01/01/2018", row2.get(0)); 
    assertEquals("23.15", row1.get(2)); 
    assertEquals("23.15", row2.get(2)); 
  }

  public void read2(String pFilePath) {
    List<String> row1 = null;
    List<String> row2 = null;
    List<String> row3 = null;
    List<String> row4 = null;
    List<String> row5 = null;
    List<String> row6 = null;
    InputStreamReader reader = null;
    try {
      reader = new InputStreamReader(new FileInputStream(pFilePath),
        Charset.forName(this.csvMethod.getCharsetName()).newDecoder());
      row1 = this.csvReader.readNextRow(null, reader, this.csvMethod);
      row2 = this.csvReader.readNextRow(null, reader, this.csvMethod);
      row3 = this.csvReader.readNextRow(null, reader, this.csvMethod);
      row4 = this.csvReader.readNextRow(null, reader, this.csvMethod);
      row5 = this.csvReader.readNextRow(null, reader, this.csvMethod);
      row6 = this.csvReader.readNextRow(null, reader, this.csvMethod);
    } catch (Exception e) {
      e.printStackTrace();
    } finally {
      if (reader != null) {
        try {
          reader.close();
        } catch (Exception e) {
          e.printStackTrace();
        }
      }
    }
    System.out.println("row1: ");
    for (String v : row1) {
      System.out.print(v + ",");
    }
    System.out.println("\nrow2: ");
    for (String v : row2) {
      System.out.print(v + ",");
    }
    System.out.println("\nrow3: ");
    for (String v : row3) {
      System.out.print(v + ",");
    }
    System.out.println("\nrow4: ");
    for (String v : row4) {
      System.out.print(v + ",");
    }
    System.out.println("\nrow5: ");
    for (String v : row5) {
      System.out.print(v + ",");
    }
    System.out.println("");
    assertEquals("Date", row1.get(0)); 
    assertEquals("Description", row1.get(1)); 
    assertEquals("ABC check #175762, ws", row2.get(1)); 
    assertEquals("ABC check #175763", row3.get(1)); 
    assertNull(row4.get(1)); 
    assertEquals("monthly fee", row5.get(1)); 
    assertEquals("23.15", row2.get(2)); 
    assertEquals("-14.15", row4.get(2)); 
    assertEquals("-11.15", row5.get(2)); 
    assertEquals("Cause", row1.get(3)); 
    assertEquals("wrong signature", row2.get(3)); 
    assertNull(row3.get(3)); 
    assertNull(row4.get(3)); 
    assertNull(row5.get(3)); 
    assertNull(row6); 
  }
}
