package org.beigesoft.service;

/*
 * Copyright (c) 2018 Beigesoft™
 *
 * Licensed under the GNU General Public License (GPL), Version 2.0
 * (the "License");
 * you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at
 *
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html
 */

import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;
import java.util.Date;
import java.math.BigDecimal;
import java.io.File;
import java.io.FileOutputStream;
import java.nio.charset.Charset;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import org.junit.Test;

import org.beigesoft.log.LoggerSimple;

import org.beigesoft.model.ECsvColumnType;
import org.beigesoft.test.persistable.PersistableHead;
import org.beigesoft.test.persistable.PersistableLine;
import org.beigesoft.test.persistable.GoodVersionTime;
import org.beigesoft.persistable.CsvMethod;
import org.beigesoft.persistable.CsvColumn;
import org.beigesoft.persistable.MatchForeign;
import org.beigesoft.persistable.MatchForeignLine;

/**
 * <p>CsvWriter test.</p>
 *
 * @author Yury Demidenko
 */
public class CsvWriterTest {

  //LoggerSimple logger = new LoggerSimple();

  CsvWriter csvWriter;

  CsvMethod csvMethod;

  MatchForeign mfp = new MatchForeign();

  public CsvWriterTest() {
    this.csvWriter = new CsvWriter();
    this.csvWriter.setUtlReflection(new UtlReflection());
    this.csvWriter.setSrvNumberToString(new SrvNumberToString());
    this.csvMethod = new CsvMethod();
    this.csvMethod.setHasHeader(true);
    this.csvMethod.setColumns(new ArrayList<CsvColumn>());
    CsvColumn colDate = new CsvColumn();
    colDate.setItsType(ECsvColumnType.DATE);
    colDate.setItsName("Date");
    colDate.setItsIndex(1);
    colDate.setSourceIndex(1);
    colDate.setDataIndex(1);
    colDate.setFieldPath("persistableHead" + "," + "itsDate");
    colDate.setDataFormat("MM/dd/yyyy");
    this.csvMethod.getColumns().add(colDate);
    CsvColumn colDesc = new CsvColumn();
    colDesc.setItsType(ECsvColumnType.STRING);
    colDesc.setItsName("Description");
    colDesc.setItsIndex(2);
    colDesc.setSourceIndex(2);
    colDesc.setDataIndex(1);
    colDesc.setTextDelimiter("\"");
    colDesc.setFieldPath("persistableHead" + "," + "tmpDescription");
    this.csvMethod.getColumns().add(colDesc);
    CsvColumn colAm = new CsvColumn();
    colAm.setItsType(ECsvColumnType.NUMERIC);
    colAm.setItsName("Total");
    colAm.setItsIndex(3);
    colAm.setSourceIndex(3);
    colAm.setDataIndex(1);
    colAm.setFieldPath("itsTotal");
    this.csvMethod.getColumns().add(colAm);
    CsvColumn colIsClosed = new CsvColumn();
    colIsClosed.setItsType(ECsvColumnType.BOOLEAN);
    colIsClosed.setItsName("IsClosed");
    colIsClosed.setItsIndex(4);
    colIsClosed.setSourceIndex(4);
    colIsClosed.setDataFormat("1,0");
    colIsClosed.setDataIndex(1);
    colIsClosed.setFieldPath("persistableHead" + "," + "isClosed");
    this.csvMethod.getColumns().add(colIsClosed);
    CsvColumn colRating = new CsvColumn();
    colRating.setItsName("Rating");
    colRating.setItsType(ECsvColumnType.INTEGER);
    colRating.setItsIndex(5);
    colRating.setSourceIndex(5);
    colRating.setDataIndex(2);
    this.csvMethod.getColumns().add(colRating);
    CsvColumn colConstant = new CsvColumn();
    colConstant.setItsName("Constant");
    colConstant.setItsType(ECsvColumnType.BOOLEAN);
    colConstant.setItsIndex(6);
    colConstant.setSourceIndex(6);
    colConstant.setConstValue("true");
    this.csvMethod.getColumns().add(colConstant);
    CsvColumn colOptional = new CsvColumn();
    colOptional.setItsName("Optional");
    colOptional.setItsType(ECsvColumnType.STRING);
    colOptional.setItsIndex(7);
    colOptional.setSourceIndex(7);
    this.csvMethod.getColumns().add(colOptional);
    CsvColumn colPrFr = new CsvColumn();
    colPrFr.setItsType(ECsvColumnType.STRING);
    colPrFr.setItsName("GoodsIdToForeign");
    colPrFr.setItsIndex(8);
    colPrFr.setSourceIndex(8);
    colPrFr.setDataIndex(1);
    colPrFr.setFieldPath("itsProduct,itsId");
    this.csvMethod.getColumns().add(colPrFr);
    mfp = new MatchForeign();
    colPrFr.setMatchForeign(mfp);
    mfp.setItsId(1L);
    mfp.setItsLines(new ArrayList<MatchForeignLine>());
    MatchForeignLine mpr1 = new MatchForeignLine();
    mpr1.setNativeVal("1");
    mpr1.setForeignVal("PR98JXPE");
    mfp.getItsLines().add(mpr1);
    MatchForeignLine mpr2 = new MatchForeignLine();
    mpr2.setNativeVal("2");
    mpr2.setForeignVal("PL98JXKL");
    mfp.getItsLines().add(mpr2);
    HldMatchForeignOnly hmf = new HldMatchForeignOnly();
    hmf.setMatchForeign(mfp);
    this.csvWriter.setHldMatchForeign(hmf);
  }

  @Test
  public void test1() throws Exception {
    List<List<Object>> data = new ArrayList<List<Object>>();
    List<Object> dataRow = new ArrayList<Object>();
    data.add(dataRow);
    List<Object> dataRow2 = new ArrayList<Object>();
    data.add(dataRow2);
    List<Object> dataRow3 = new ArrayList<Object>();
    data.add(dataRow3);
    PersistableLine pl = new PersistableLine();
    PersistableHead ph = new PersistableHead();
    ph.setTmpDescription("sample a, sample b...");
    ph.setItsDate(new Date());
    ph.setIsClosed(true);
    pl.setPersistableHead(ph);
    GoodVersionTime pr1 = new GoodVersionTime();
    pr1.setItsId(1L);
    pl.setItsProduct(pr1);
    pl.setItsTotal(new BigDecimal("12.22"));
    dataRow.add(pl); 
    dataRow.add(Integer.valueOf(2));
    PersistableLine pl2 = new PersistableLine();
    PersistableHead ph2 = new PersistableHead();
    ph2.setItsDate(new Date());
    pl2.setPersistableHead(ph2);
    GoodVersionTime pr2 = new GoodVersionTime();
    pr2.setItsId(2L);
    pl2.setItsProduct(pr2);
    pl2.setItsTotal(new BigDecimal("52.22"));
    dataRow2.add(pl2);
    dataRow2.add(Integer.valueOf(5));
    dataRow3.add(null);
    dataRow3.add(null);
    File file = new File("persistableLines.csv");
    this.csvWriter.write(null, data, this.csvMethod, new FileOutputStream(file));
  }
}
