package org.beigesoft.service;

/*
 * Copyright (c) 2018 Beigesoft ™
 *
 * Licensed under the GNU General Public License (GPL), Version 2.0
 * (the "License");
 * you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at
 *
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html
 */

import java.util.Locale;

import static org.junit.Assert.*;
import org.junit.Test;

import org.beigesoft.log.ILogger;
import org.beigesoft.log.LoggerSimple;

/**
 * <p>I18N tests.</p>
 *
 * @author Yury Demidenko
 */
public class I18NTest {
  
  private ILogger log;

  public void initIfNeed() {
    if (this.log == null) {
      this.log = new LoggerSimple();
    }
  }

  @Test
  public void tst1(){
    initIfNeed();
    SrvI18n srvI18n = new SrvI18n();
    srvI18n.setLogger(this.log);
    srvI18n.initDefault();
    srvI18n.add(new String[]{"en", "US", "ru", "RU"});
    this.log.debug(null, I18NTest.class, "Default locale country/language: " + Locale.getDefault().getCountry()
      + "/" + Locale.getDefault().getLanguage());
    this.log.debug(null, I18NTest.class, "Yes en = " + srvI18n.getMsg("Yes", "en"));
    this.log.debug(null, I18NTest.class, "Yes en = " + srvI18n.getMsg("Yes", "en"));
    this.log.debug(null, I18NTest.class, "Omitted en = " + srvI18n.getMsg("Omitted", "en"));
    this.log.debug(null, I18NTest.class, "Yes ru = " + srvI18n.getMsg("Yes", "ru"));
    this.log.debug(null, I18NTest.class, "Omitted ru = " + srvI18n.getMsg("Omitted", "ru"));
    this.log.debug(null, I18NTest.class, "Omitted def = " + srvI18n.getMsg("Omitted"));
    assertEquals(2, srvI18n.getMessagesMap().size());
    assertEquals("Yes", srvI18n.getMsg("Yes", "en"));
    assertEquals("Да", srvI18n.getMsg("Yes", "ru"));
  }

  /**
   * <p>Getter for log.</p>
   * @return ILogger
   **/
  public final ILogger getLog() {
    return this.log;
  }

  /**
   * <p>Setter for log.</p>
   * @param pLog reference
   **/
  public final void setLog(final ILogger pLog) {
    this.log = pLog;
  }
}
